@@ -4,7 +4,7 @@
 // Author:      Vaclav Slavik
 // Modified by:
 // Created:     04/04/2003
-// RCS-ID:      $Id: taskbarx11.h,v 1.5 2004/05/29 17:01:18 VS Exp $
+// RCS-ID:      $Id: taskbarx11.h,v 1.6 2005/09/23 12:50:53 MR Exp $
 // Copyright:   (c) Vaclav Slavik, 2003
 // Licence:     wxWindows licence
 /////////////////////////////////////////////////////////////////////////
@@ -12,10 +12,6 @@
 #ifndef _TASKBAR_H_
 #define _TASKBAR_H_
 
-#if defined(__GNUG__) && !defined(NO_GCC_PRAGMA)
-#pragma interface "taskbarx11.h"
-#endif
-
 class WXDLLEXPORT wxIcon;
 class WXDLLEXPORT wxFrame;
 class WXDLLEXPORT wxWindow;
