@@ -1,9 +1,9 @@
 /////////////////////////////////////////////////////////////////////////////
-// Name:        xh_chckl.h
+// Name:        wx/xrc/xh_chckl.h
 // Purpose:     XML resource handler for wxCheckListBox
 // Author:      Bob Mitchell
 // Created:     2000/03/21
-// RCS-ID:      $Id: xh_chckl.h,v 1.7 2005/05/22 00:29:17 MW Exp $
+// RCS-ID:      $Id: xh_chckl.h,v 1.10 2006/10/03 14:53:30 VZ Exp $
 // Copyright:   (c) 2000 Bob Mitchell and Verant Interactive
 // Licence:     wxWindows licence
 /////////////////////////////////////////////////////////////////////////////
@@ -11,26 +11,24 @@
 #ifndef _WX_XH_CHCKL_H_
 #define _WX_XH_CHCKL_H_
 
-#if defined(__GNUG__) && !defined(NO_GCC_PRAGMA)
-#pragma interface "xh_chckl.h"
-#endif
-
 #include "wx/xrc/xmlres.h"
 
-#if wxUSE_CHECKLISTBOX
+#if wxUSE_XRC && wxUSE_CHECKLISTBOX
 
 class WXDLLIMPEXP_XRC wxCheckListBoxXmlHandler : public wxXmlResourceHandler
 {
-DECLARE_DYNAMIC_CLASS(wxCheckListBoxXmlHandler)
+    DECLARE_DYNAMIC_CLASS(wxCheckListBoxXmlHandler)
+
 public:
     wxCheckListBoxXmlHandler();
     virtual wxObject *DoCreateResource();
     virtual bool CanHandle(wxXmlNode *node);
+
 private:
     bool m_insideBox;
     wxArrayString strList;
 };
 
-#endif
+#endif // wxUSE_XRC && wxUSE_CHECKLISTBOX
 
 #endif // _WX_XH_CHECKLIST_H_
