@@ -1,9 +1,9 @@
 /////////////////////////////////////////////////////////////////////////////
-// Name:        xh_unkwn.h
+// Name:        wx/xrc/xh_unkwn.h
 // Purpose:     XML resource handler for unkown widget
 // Author:      Vaclav Slavik
 // Created:     2000/03/05
-// RCS-ID:      $Id: xh_unkwn.h,v 1.5 2005/01/07 21:27:20 VS Exp $
+// RCS-ID:      $Id: xh_unkwn.h,v 1.8 2006/10/03 14:53:32 VZ Exp $
 // Copyright:   (c) 2000 Vaclav Slavik
 // Licence:     wxWindows licence
 /////////////////////////////////////////////////////////////////////////////
@@ -11,21 +11,20 @@
 #ifndef _WX_XH_UNKWN_H_
 #define _WX_XH_UNKWN_H_
 
-#if defined(__GNUG__) && !defined(NO_GCC_PRAGMA)
-#pragma interface "xh_unkwn.h"
-#endif
-
 #include "wx/xrc/xmlres.h"
 
+#if wxUSE_XRC
 
 class WXDLLIMPEXP_XRC wxUnknownWidgetXmlHandler : public wxXmlResourceHandler
 {
-DECLARE_DYNAMIC_CLASS(wxUnknownWidgetXmlHandler)
+    DECLARE_DYNAMIC_CLASS(wxUnknownWidgetXmlHandler)
+
 public:
     wxUnknownWidgetXmlHandler();
     virtual wxObject *DoCreateResource();
     virtual bool CanHandle(wxXmlNode *node);
 };
 
+#endif // wxUSE_XRC
 
 #endif // _WX_XH_UNKWN_H_
