@@ -4,26 +4,21 @@
 // Author:      Julian Smart
 // Modified by:
 // Created:     04/01/98
-// RCS-ID:      $Id: treetest.cpp,v 1.99 2005/05/21 03:03:40 KH Exp $
+// RCS-ID:      $Id: treetest.cpp,v 1.111 2007/01/07 15:43:22 VZ Exp $
 // Copyright:   (c) Julian Smart
 // Licence:     wxWindows license
 /////////////////////////////////////////////////////////////////////////////
 
-#ifdef __GNUG__
-  #pragma implementation
-  #pragma interface
-#endif
-
 // For compilers that support precompilation, includes "wx/wx.h".
 #include "wx/wxprec.h"
 
 #ifdef __BORLANDC__
-  #pragma hdrstop
+    #pragma hdrstop
 #endif
 
 #ifndef WX_PRECOMP
-  #include "wx/wx.h"
-  #include "wx/log.h"
+    #include "wx/wx.h"
+    #include "wx/log.h"
 #endif
 
 #include "wx/colordlg.h"
@@ -47,11 +42,10 @@
 #include "icon4.xpm"
 #include "icon5.xpm"
 
-#if defined(__WXGTK__) || defined(__WXX11__) || defined(__WXMOTIF__) || defined(__WXMAC__) || defined(__WXMGL__)
-#include "mondrian.xpm"
+#ifndef __WXMSW__
+    #include "../sample.xpm"
 #endif
 
-
 // verify that the item is ok and insult the user if it is not
 #define CHECK_ITEM( item ) if ( !item.IsOk() ) {                                 \
                              wxMessageBox(wxT("Please select some item first!"), \
@@ -82,6 +76,7 @@ BEGIN_EVENT_TABLE(MyFrame, wxFrame)
     MENU_LINK(SetBgColour)
     MENU_LINK(ResetStyle)
 
+    MENU_LINK(Highlight)
     MENU_LINK(Dump)
 #ifndef NO_MULTIPLE_SELECTION
     MENU_LINK(DumpSelected)
@@ -184,7 +179,7 @@ MyFrame::MyFrame(const wxString& title, int x, int y, int w, int h)
     SetBackgroundColour(wxColour(255, 255, 255));
 
     // Give it an icon
-    SetIcon(wxICON(mondrian));
+    SetIcon(wxICON(sample));
 
 #if wxUSE_MENUS
     // Make a menubar
@@ -266,9 +261,11 @@ MyFrame::MyFrame(const wxString& title, int x, int y, int w, int h)
     SetMenuBar(menu_bar);
 #endif // wxUSE_MENUS
 
+    m_panel = new wxPanel(this);
+
 #if wxUSE_LOG
     // create the controls
-    m_textCtrl = new wxTextCtrl(this, wxID_ANY, wxT(""),
+    m_textCtrl = new wxTextCtrl(m_panel, wxID_ANY, wxT(""),
                                 wxDefaultPosition, wxDefaultSize,
                                 wxTE_MULTILINE | wxSUNKEN_BORDER);
 #endif // wxUSE_LOG
@@ -327,7 +324,7 @@ void MyFrame::CreateTreeWithDefStyle()
 
 void MyFrame::CreateTree(long style)
 {
-    m_treeCtrl = new MyTreeCtrl(this, TreeTest_Ctrl,
+    m_treeCtrl = new MyTreeCtrl(m_panel, TreeTest_Ctrl,
                                 wxDefaultPosition, wxDefaultSize,
                                 style);
     Resize();
@@ -460,6 +457,26 @@ void MyFrame::DoSort(bool reverse)
     m_treeCtrl->DoSortChildren(item, reverse);
 }
 
+void MyFrame::OnHighlight(wxCommandEvent& WXUNUSED(event))
+{
+    wxTreeItemId id = m_treeCtrl->GetSelection();
+
+    CHECK_ITEM( id );
+
+    wxRect r;
+    if ( !m_treeCtrl->GetBoundingRect(id, r, true /* text, not full row */) )
+    {
+        wxLogMessage(_T("Failed to get bounding item rect"));
+        return;
+    }
+
+    wxClientDC dc(m_treeCtrl);
+    dc.SetBrush(*wxRED);
+    dc.SetPen(*wxTRANSPARENT_PEN);
+    dc.DrawRectangle(r);
+    m_treeCtrl->Update();
+}
+
 void MyFrame::OnDump(wxCommandEvent& WXUNUSED(event))
 {
     wxTreeItemId root = m_treeCtrl->GetSelection();
@@ -481,7 +498,7 @@ void MyFrame::OnDumpSelected(wxCommandEvent& WXUNUSED(event))
     wxArrayTreeItemIds array;
 
     size_t count = m_treeCtrl->GetSelections(array);
-    wxLogMessage(wxT("%u items selected"), count);
+    wxLogMessage(wxT("%u items selected"), unsigned(count));
 
     for ( size_t n = 0; n < count; n++ )
     {
@@ -794,9 +811,9 @@ void MyTreeCtrl::AddItemsRecursively(const wxTreeItemId& idParent,
         {
             // at depth 1 elements won't have any more children
             if ( hasChildren )
-                str.Printf(wxT("%s child %d"), wxT("Folder"), n + 1);
+                str.Printf(wxT("%s child %u"), wxT("Folder"), unsigned(n + 1));
             else
-                str.Printf(wxT("%s child %d.%d"), wxT("File"), folder, n + 1);
+                str.Printf(wxT("%s child %u.%u"), wxT("File"), unsigned(folder), unsigned(n + 1));
 
             // here we pass to AppendItem() normal and selected item images (we
             // suppose that selected image follows the normal one in the enum)
@@ -883,18 +900,33 @@ void MyTreeCtrl::GetItemsRecursively(const wxTreeItemId& idParent,
 
 void MyTreeCtrl::DoToggleIcon(const wxTreeItemId& item)
 {
-    int image = GetItemImage(item) == TreeCtrlIcon_Folder ? TreeCtrlIcon_File
-                                                          : TreeCtrlIcon_Folder;
+    int image = (GetItemImage(item) == TreeCtrlIcon_Folder)
+                    ? TreeCtrlIcon_File
+                    : TreeCtrlIcon_Folder;
+    SetItemImage(item, image, wxTreeItemIcon_Normal);
 
-    SetItemImage(item, image);
+    image = (GetItemImage(item) == TreeCtrlIcon_FolderSelected)
+                    ? TreeCtrlIcon_FileSelected
+                    : TreeCtrlIcon_FolderSelected;
+    SetItemImage(item, image, wxTreeItemIcon_Selected);
 }
 
+void MyTreeCtrl::LogEvent(const wxChar *name, const wxTreeEvent& event)
+{
+    wxTreeItemId item = event.GetItem();
+    wxString text;
+    if ( item.IsOk() )
+        text << _T('"') << GetItemText(item).c_str() << _T('"');
+    else
+        text = _T("invalid item");
+    wxLogMessage(wxT("%s(%s)"), name, text.c_str());
+}
 
 // avoid repetition
 #define TREE_EVENT_HANDLER(name)                                 \
 void MyTreeCtrl::name(wxTreeEvent& event)                        \
 {                                                                \
-    wxLogMessage(wxT(#name));                                    \
+    LogEvent(_T(#name), event);                                  \
     SetLastItem(wxTreeItemId());                                 \
     event.Skip();                                                \
 }
@@ -936,8 +968,6 @@ void LogKeyEvent(const wxChar *name, const wxKeyEvent& event)
             case WXK_MENU: key = wxT("MENU"); break;
             case WXK_PAUSE: key = wxT("PAUSE"); break;
             case WXK_CAPITAL: key = wxT("CAPITAL"); break;
-            case WXK_PRIOR: key = wxT("PRIOR"); break;
-            case WXK_NEXT: key = wxT("NEXT"); break;
             case WXK_END: key = wxT("END"); break;
             case WXK_HOME: key = wxT("HOME"); break;
             case WXK_LEFT: key = wxT("LEFT"); break;
@@ -1006,7 +1036,6 @@ void LogKeyEvent(const wxChar *name, const wxKeyEvent& event)
             case WXK_NUMPAD_UP: key = wxT("NUMPAD_UP"); break;
             case WXK_NUMPAD_RIGHT: key = wxT("NUMPAD_RIGHT"); break;
             case WXK_NUMPAD_DOWN: key = wxT("NUMPAD_DOWN"); break;
-            case WXK_NUMPAD_PRIOR: key = wxT("NUMPAD_PRIOR"); break;
             case WXK_NUMPAD_PAGEUP: key = wxT("NUMPAD_PAGEUP"); break;
             case WXK_NUMPAD_PAGEDOWN: key = wxT("NUMPAD_PAGEDOWN"); break;
             case WXK_NUMPAD_END: key = wxT("NUMPAD_END"); break;
@@ -1055,8 +1084,12 @@ void MyTreeCtrl::OnBeginDrag(wxTreeEvent& event)
     {
         m_draggedItem = event.GetItem();
 
-        wxLogMessage(wxT("OnBeginDrag: started dragging %s"),
-                     GetItemText(m_draggedItem).c_str());
+        wxPoint clientpt = event.GetPoint();
+        wxPoint screenpt = ClientToScreen(clientpt);
+
+        wxLogMessage(wxT("OnBeginDrag: started dragging %s at screen coords (%i,%i)"),
+                     GetItemText(m_draggedItem).c_str(),
+                     screenpt.x, screenpt.y);
 
         event.Allow();
     }
@@ -1166,43 +1199,21 @@ void MyTreeCtrl::OnItemMenu(wxTreeEvent& event)
     wxTreeItemId itemId = event.GetItem();
     MyTreeItemData *item = itemId.IsOk() ? (MyTreeItemData *)GetItemData(itemId)
                                          : NULL;
+    wxPoint clientpt = event.GetPoint();
+    wxPoint screenpt = ClientToScreen(clientpt);
 
-    wxLogMessage(wxT("OnItemMenu for item \"%s\""), item ? item->GetDesc()
-                                                         : _T(""));
+    wxLogMessage(wxT("OnItemMenu for item \"%s\" at screen coords (%i, %i)"), 
+                 item ? item->GetDesc() : _T(""), screenpt.x, screenpt.y);
 
+    ShowMenu(itemId, clientpt);
     event.Skip();
 }
 
 void MyTreeCtrl::OnContextMenu(wxContextMenuEvent& event)
 {
     wxPoint pt = event.GetPosition();
-    wxTreeItemId item;
-    if ( !HasFlag(wxTR_MULTIPLE) )
-        item = GetSelection();
+
     wxLogMessage(wxT("OnContextMenu at screen coords (%i, %i)"), pt.x, pt.y);
-
-    // check if event was generated by keyboard (MSW-specific?)
-    if ( pt.x == -1 && pt.y == -1 ) //(this is how MSW indicates it)
-    {
-        // attempt to guess where to show the menu
-        if ( item.IsOk() )
-        {
-            // if an item was clicked, show menu to the right of it
-            wxRect rect;
-            GetBoundingRect(item, rect, true /* only the label */);
-            pt = wxPoint(rect.GetRight(), rect.GetTop());
-        }
-        else
-        {
-            pt = wxPoint(0, 0);
-        }
-    }
-    else // event was generated by mouse, use supplied coords
-    {
-        pt = ScreenToClient(pt);
-    }
-
-    ShowMenu(item, pt);
 }
 
 void MyTreeCtrl::ShowMenu(wxTreeItemId id, const wxPoint& pt)
@@ -1220,6 +1231,8 @@ void MyTreeCtrl::ShowMenu(wxTreeItemId id, const wxPoint& pt)
 #if wxUSE_MENUS
     wxMenu menu(title);
     menu.Append(TreeTest_About, wxT("&About..."));
+    menu.AppendSeparator();
+    menu.Append(TreeTest_Highlight, wxT("&Highlight item"));
     menu.Append(TreeTest_Dump, wxT("&Dump"));
 
     PopupMenu(&menu, pt);
@@ -1280,6 +1293,6 @@ void MyTreeItemData::ShowInfo(wxTreeCtrl *tree)
                  Bool2String(tree->IsSelected(GetId())),
                  Bool2String(tree->IsExpanded(GetId())),
                  Bool2String(tree->IsBold(GetId())),
-                 tree->GetChildrenCount(GetId()),
-                 tree->GetChildrenCount(GetId(), false));
+                 unsigned(tree->GetChildrenCount(GetId())),
+                 unsigned(tree->GetChildrenCount(GetId(), false)));
 }
