@@ -1,23 +1,19 @@
 //////////////////////////////////////////////////////////////////////////////
-// Name:        filedlgg.cpp
+// Name:        src/generic/filedlgg.cpp
 // Purpose:     wxGenericFileDialog
 // Author:      Robert Roebling
 // Modified by:
 // Created:     12/12/98
-// RCS-ID:      $Id: filedlgg.cpp,v 1.143.2.2 2006/03/06 10:44:48 JS Exp $
+// RCS-ID:      $Id: filedlgg.cpp,v 1.177 2006/10/27 08:50:25 RR Exp $
 // Copyright:   (c) Robert Roebling
 // Licence:     wxWindows licence
 /////////////////////////////////////////////////////////////////////////////
 
-#if defined(__GNUG__) && !defined(NO_GCC_PRAGMA)
-#pragma implementation "filedlgg.h"
-#endif
-
 // For compilers that support precompilation, includes "wx.h".
 #include "wx/wxprec.h"
 
 #ifdef __BORLANDC__
-#pragma hdrstop
+    #pragma hdrstop
 #endif
 
 #if wxUSE_FILEDLG
@@ -27,26 +23,32 @@
 #error wxGenericFileDialog currently only supports Unix, win32 and DOS
 #endif
 
-#include "wx/checkbox.h"
-#include "wx/textctrl.h"
-#include "wx/choice.h"
-#include "wx/checkbox.h"
-#include "wx/stattext.h"
-#include "wx/debug.h"
-#include "wx/log.h"
-#include "wx/intl.h"
-#include "wx/msgdlg.h"
-#include "wx/sizer.h"
-#include "wx/bmpbuttn.h"
+#ifndef WX_PRECOMP
+    #ifdef __WXMSW__
+        #include "wx/msw/wrapwin.h"
+    #endif
+    #include "wx/hash.h"
+    #include "wx/intl.h"
+    #include "wx/settings.h"
+    #include "wx/log.h"
+    #include "wx/msgdlg.h"
+    #include "wx/bmpbuttn.h"
+    #include "wx/checkbox.h"
+    #include "wx/choice.h"
+    #include "wx/stattext.h"
+    #include "wx/textctrl.h"
+    #include "wx/sizer.h"
+    #include "wx/filedlg.h"     // wxFD_OPEN, wxFD_SAVE...
+#endif
+
+#include "wx/longlong.h"
 #include "wx/tokenzr.h"
 #include "wx/config.h"
 #include "wx/imaglist.h"
 #include "wx/dir.h"
 #include "wx/artprov.h"
-#include "wx/settings.h"
 #include "wx/filefn.h"
 #include "wx/file.h"        // for wxS_IXXX constants only
-#include "wx/filedlg.h"     // wxOPEN, wxSAVE...
 #include "wx/generic/filedlgg.h"
 #include "wx/generic/dirctrlg.h" // for wxFileIconsTable
 
@@ -55,8 +57,8 @@
 #endif
 
 #ifndef __WXWINCE__
-#include <sys/types.h>
-#include <sys/stat.h>
+    #include <sys/types.h>
+    #include <sys/stat.h>
 #endif
 
 #ifdef __UNIX__
@@ -68,7 +70,6 @@
 #endif
 
 #ifdef __WINDOWS__
-    #include "wx/msw/wrapwin.h"
     #include "wx/msw/mslu.h"
 #endif
 
@@ -89,56 +90,83 @@
 // ----------------------------------------------------------------------------
 
 static
-int wxCALLBACK wxFileDataNameCompare( long data1, long data2, long data)
+int wxCALLBACK wxFileDataNameCompare( long data1, long data2, long sortOrder)
 {
-     wxFileData *fd1 = (wxFileData*)data1;
-     wxFileData *fd2 = (wxFileData*)data2;
-     if (fd1->GetFileName() == wxT("..")) return -data;
-     if (fd2->GetFileName() == wxT("..")) return data;
-     if (fd1->IsDir() && !fd2->IsDir()) return -data;
-     if (fd2->IsDir() && !fd1->IsDir()) return data;
-     return data*wxStrcmp( fd1->GetFileName(), fd2->GetFileName() );
+     wxFileData *fd1 = (wxFileData *)wxUIntToPtr(data1);
+     wxFileData *fd2 = (wxFileData *)wxUIntToPtr(data2);
+
+     if (fd1->GetFileName() == wxT(".."))
+         return -sortOrder;
+     if (fd2->GetFileName() == wxT(".."))
+         return sortOrder;
+     if (fd1->IsDir() && !fd2->IsDir())
+         return -sortOrder;
+     if (fd2->IsDir() && !fd1->IsDir())
+         return sortOrder;
+
+     return sortOrder*wxStrcmp( fd1->GetFileName(), fd2->GetFileName() );
 }
 
 static
-int wxCALLBACK wxFileDataSizeCompare( long data1, long data2, long data)
+int wxCALLBACK wxFileDataSizeCompare(long data1, long data2, long sortOrder)
 {
-     wxFileData *fd1 = (wxFileData*)data1;
-     wxFileData *fd2 = (wxFileData*)data2;
-     if (fd1->GetFileName() == wxT("..")) return -data;
-     if (fd2->GetFileName() == wxT("..")) return data;
-     if (fd1->IsDir() && !fd2->IsDir()) return -data;
-     if (fd2->IsDir() && !fd1->IsDir()) return data;
-     if (fd1->IsLink() && !fd2->IsLink()) return -data;
-     if (fd2->IsLink() && !fd1->IsLink()) return data;
-     return data*(fd1->GetSize() - fd2->GetSize());
+     wxFileData *fd1 = (wxFileData *)wxUIntToPtr(data1);
+     wxFileData *fd2 = (wxFileData *)wxUIntToPtr(data2);
+
+     if (fd1->GetFileName() == wxT(".."))
+         return -sortOrder;
+     if (fd2->GetFileName() == wxT(".."))
+         return sortOrder;
+     if (fd1->IsDir() && !fd2->IsDir())
+         return -sortOrder;
+     if (fd2->IsDir() && !fd1->IsDir())
+         return sortOrder;
+     if (fd1->IsLink() && !fd2->IsLink())
+         return -sortOrder;
+     if (fd2->IsLink() && !fd1->IsLink())
+         return sortOrder;
+
+     return fd1->GetSize() > fd2->GetSize() ? sortOrder : -sortOrder;
 }
 
 static
-int wxCALLBACK wxFileDataTypeCompare( long data1, long data2, long data)
+int wxCALLBACK wxFileDataTypeCompare(long data1, long data2, long sortOrder)
 {
-     wxFileData *fd1 = (wxFileData*)data1;
-     wxFileData *fd2 = (wxFileData*)data2;
-     if (fd1->GetFileName() == wxT("..")) return -data;
-     if (fd2->GetFileName() == wxT("..")) return data;
-     if (fd1->IsDir() && !fd2->IsDir()) return -data;
-     if (fd2->IsDir() && !fd1->IsDir()) return data;
-     if (fd1->IsLink() && !fd2->IsLink()) return -data;
-     if (fd2->IsLink() && !fd1->IsLink()) return data;
-     return data*wxStrcmp( fd1->GetFileType(), fd2->GetFileType() );
+     wxFileData *fd1 = (wxFileData *)wxUIntToPtr(data1);
+     wxFileData *fd2 = (wxFileData *)wxUIntToPtr(data2);
+
+     if (fd1->GetFileName() == wxT(".."))
+         return -sortOrder;
+     if (fd2->GetFileName() == wxT(".."))
+         return sortOrder;
+     if (fd1->IsDir() && !fd2->IsDir())
+         return -sortOrder;
+     if (fd2->IsDir() && !fd1->IsDir())
+         return sortOrder;
+     if (fd1->IsLink() && !fd2->IsLink())
+         return -sortOrder;
+     if (fd2->IsLink() && !fd1->IsLink())
+         return sortOrder;
+
+     return sortOrder*wxStrcmp( fd1->GetFileType(), fd2->GetFileType() );
 }
 
 static
-int wxCALLBACK wxFileDataTimeCompare( long data1, long data2, long data)
+int wxCALLBACK wxFileDataTimeCompare(long data1, long data2, long sortOrder)
 {
-     wxFileData *fd1 = (wxFileData*)data1;
-     wxFileData *fd2 = (wxFileData*)data2;
-     if (fd1->GetFileName() == wxT("..")) return -data;
-     if (fd2->GetFileName() == wxT("..")) return data;
-     if (fd1->IsDir() && !fd2->IsDir()) return -data;
-     if (fd2->IsDir() && !fd1->IsDir()) return data;
+     wxFileData *fd1 = (wxFileData *)wxUIntToPtr(data1);
+     wxFileData *fd2 = (wxFileData *)wxUIntToPtr(data2);
 
-     return fd1->GetDateTime().IsLaterThan(fd2->GetDateTime()) ? int(data) : -int(data);
+     if (fd1->GetFileName() == wxT(".."))
+         return -sortOrder;
+     if (fd2->GetFileName() == wxT(".."))
+         return sortOrder;
+     if (fd1->IsDir() && !fd2->IsDir())
+         return -sortOrder;
+     if (fd2->IsDir() && !fd1->IsDir())
+         return sortOrder;
+
+     return fd1->GetDateTime().IsLaterThan(fd2->GetDateTime()) ? sortOrder : -sortOrder;
 }
 
 #if defined(__WXWINCE__)
@@ -226,7 +254,7 @@ void wxFileData::ReadData()
             NULL);
 
     if (fileHandle != INVALID_HANDLE_VALUE)
-    { 
+    {
         m_size = GetFileSize(fileHandle, 0);
         CloseHandle(fileHandle);
     }
@@ -254,7 +282,8 @@ void wxFileData::ReadData()
 
     m_type |= (buff.st_mode & S_IFDIR) != 0 ? is_dir : 0;
     m_type |= (buff.st_mode & wxS_IXUSR) != 0 ? is_exe : 0;
-    m_size = (long)buff.st_size;
+
+    m_size = buff.st_size;
 
     m_dateTime = buff.st_mtime;
 #endif
@@ -272,7 +301,7 @@ void wxFileData::ReadData()
                          buff.st_mode & wxS_IWOTH ? _T('w') : _T('-'),
                          buff.st_mode & wxS_IXOTH ? _T('x') : _T('-'));
 #elif defined(__WIN32__)
-    DWORD attribs = GetFileAttributes(m_filePath.fn_str());
+    DWORD attribs = ::GetFileAttributes(m_filePath.c_str());
     if (attribs != (DWORD)-1)
     {
         m_permissions.Printf(_T("%c%c%c%c"),
@@ -328,7 +357,8 @@ wxString wxFileData::GetHint() const
     else if (IsDrive())
         s += _("<DRIVE>");
     else // plain file
-        s += wxString::Format( _("%ld bytes"), m_size );
+        s += wxString::Format(wxPLURAL("%ld byte", "%ld bytes", m_size),
+                              wxLongLong(m_size).ToString().c_str());
 
     s += wxT(' ');
 
@@ -340,7 +370,7 @@ wxString wxFileData::GetHint() const
     }
 
     return s;
-};
+}
 
 wxString wxFileData::GetEntry( fileListFieldType num ) const
 {
@@ -353,7 +383,7 @@ wxString wxFileData::GetEntry( fileListFieldType num ) const
 
         case FileList_Size:
             if (!IsDir() && !IsLink() && !IsDrive())
-                s.Printf(_T("%ld"), m_size);
+                s = wxLongLong(m_size).ToString();
             break;
 
         case FileList_Type:
@@ -401,7 +431,7 @@ void wxFileData::MakeItem( wxListItem &item )
         if ( dg.Ok() )
             item.SetTextColour(dg);
     }
-    item.m_data = (long)this;
+    item.m_data = wxPtrToUInt(this);
 }
 
 //-----------------------------------------------------------------------------
@@ -693,7 +723,7 @@ void wxFileCtrl::MakeDir()
     if (id != -1)
     {
         SortItems(m_sort_field, m_sort_foward);
-        id = FindItem( 0, (long)fd );
+        id = FindItem( 0, wxPtrToUInt(fd) );
         EnsureVisible( id );
         EditLabel( id );
     }
@@ -705,7 +735,7 @@ void wxFileCtrl::GoToParentDir()
 {
     if (!IsTopMostDir(m_dirName))
     {
-        size_t len = m_dirName.Len();
+        size_t len = m_dirName.length();
         if (wxEndsWithPathSeparator(m_dirName))
             m_dirName.Remove( len-1, 1 );
         wxString fname( wxFileNameFromPath(m_dirName) );
@@ -793,8 +823,8 @@ void wxFileCtrl::OnListEndLabelEdit( wxListEvent &event )
     wxASSERT( fd );
 
     if ((event.GetLabel().empty()) ||
-        (event.GetLabel() == _(".")) ||
-        (event.GetLabel() == _("..")) ||
+        (event.GetLabel() == wxT(".")) ||
+        (event.GetLabel() == wxT("..")) ||
         (event.GetLabel().First( wxFILE_SEP_PATH ) != wxNOT_FOUND))
     {
         wxMessageDialog dialog(this, _("Illegal directory name."), _("Error"), wxOK | wxICON_ERROR );
@@ -856,35 +886,30 @@ void wxFileCtrl::OnListColClick( wxListEvent &event )
     SortItems(m_sort_field, m_sort_foward);
 }
 
-void wxFileCtrl::SortItems(wxFileData::fileListFieldType field, bool foward)
+void wxFileCtrl::SortItems(wxFileData::fileListFieldType field, bool forward)
 {
     m_sort_field = field;
-    m_sort_foward = foward;
-    long sort_dir = foward ? 1 : -1;
+    m_sort_foward = forward;
+    const long sort_dir = forward ? 1 : -1;
 
     switch (m_sort_field)
     {
-        case wxFileData::FileList_Name :
-        {
-            wxListCtrl::SortItems((wxListCtrlCompare)wxFileDataNameCompare, sort_dir);
-            break;
-        }
         case wxFileData::FileList_Size :
-        {
-             wxListCtrl::SortItems((wxListCtrlCompare)wxFileDataSizeCompare, sort_dir);
+            wxListCtrl::SortItems(wxFileDataSizeCompare, sort_dir);
             break;
-        }
+
         case wxFileData::FileList_Type :
-        {
-             wxListCtrl::SortItems((wxListCtrlCompare)wxFileDataTypeCompare, sort_dir);
-             break;
-        }
+            wxListCtrl::SortItems(wxFileDataTypeCompare, sort_dir);
+            break;
+
         case wxFileData::FileList_Time :
-        {
-             wxListCtrl::SortItems((wxListCtrlCompare)wxFileDataTimeCompare, sort_dir);
-             break;
-        }
-        default : break;
+            wxListCtrl::SortItems(wxFileDataTimeCompare, sort_dir);
+            break;
+
+        case wxFileData::FileList_Name :
+        default :
+            wxListCtrl::SortItems(wxFileDataNameCompare, sort_dir);
+            break;
     }
 }
 
@@ -950,10 +975,12 @@ wxGenericFileDialog::wxGenericFileDialog(wxWindow *parent,
                            const wxString& wildCard,
                            long  style,
                            const wxPoint& pos,
+                           const wxSize& sz,
+                           const wxString& name,
                            bool  bypassGenericImpl ) : wxFileDialogBase()
 {
     Init();
-    Create( parent, message, defaultDir, defaultFile, wildCard, style, pos, bypassGenericImpl );
+    Create( parent, message, defaultDir, defaultFile, wildCard, style, pos, sz, name, bypassGenericImpl );
 }
 
 bool wxGenericFileDialog::Create( wxWindow *parent,
@@ -963,12 +990,14 @@ bool wxGenericFileDialog::Create( wxWindow *parent,
                                   const wxString& wildCard,
                                   long  style,
                                   const wxPoint& pos,
+                                  const wxSize& sz,
+                                  const wxString& name,
                                   bool  bypassGenericImpl )
 {
     m_bypassGenericImpl = bypassGenericImpl;
 
     if (!wxFileDialogBase::Create(parent, message, defaultDir, defaultFile,
-                                  wildCard, style, pos))
+                                  wildCard, style, pos, sz, name))
     {
         return false;
     }
@@ -976,8 +1005,9 @@ bool wxGenericFileDialog::Create( wxWindow *parent,
     if (m_bypassGenericImpl)
         return true;
 
-    if (!wxDialog::Create( parent, wxID_ANY, message, pos, wxDefaultSize,
-                           wxDEFAULT_DIALOG_STYLE | wxRESIZE_BORDER ))
+    if (!wxDialog::Create( parent, wxID_ANY, message, pos, sz,
+                           wxDEFAULT_DIALOG_STYLE | wxRESIZE_BORDER | style, name
+                           ))
     {
         return false;
     }
@@ -992,11 +1022,6 @@ bool wxGenericFileDialog::Create( wxWindow *parent,
                               &ms_lastShowHidden);
     }
 
-    if (m_dialogStyle == 0)
-        m_dialogStyle = wxOPEN;
-    if ((m_dialogStyle & wxMULTIPLE ) && !(m_dialogStyle & wxOPEN))
-        m_dialogStyle |= wxOPEN;
-
     if ((m_dir.empty()) || (m_dir == wxT(".")))
     {
         m_dir = wxGetCwd();
@@ -1004,7 +1029,7 @@ bool wxGenericFileDialog::Create( wxWindow *parent,
             m_dir = wxFILE_SEP_PATH;
     }
 
-    size_t len = m_dir.Len();
+    size_t len = m_dir.length();
     if ((len > 1) && (wxEndsWithPathSeparator(m_dir)))
         m_dir.Remove( len-1, 1 );
 
@@ -1070,14 +1095,14 @@ bool wxGenericFileDialog::Create( wxWindow *parent,
         mainsizer->Add( buttonsizer, 0, wxALL | wxEXPAND, 5 );
 
     wxBoxSizer *staticsizer = new wxBoxSizer( wxHORIZONTAL );
-    if (!is_pda)
+    if (is_pda)
         staticsizer->Add( new wxStaticText( this, wxID_ANY, _("Current directory:") ), 0, wxRIGHT, 10 );
     m_static = new wxStaticText( this, wxID_ANY, m_dir );
     staticsizer->Add( m_static, 1 );
     mainsizer->Add( staticsizer, 0, wxEXPAND | wxLEFT|wxRIGHT|wxBOTTOM, 10 );
 
     long style2 = ms_lastViewStyle;
-    if ( !(m_dialogStyle & wxMULTIPLE) )
+    if ( !HasFdFlag(wxFD_MULTIPLE) )
         style2 |= wxLC_SINGLE_SEL;
 
 #ifdef __WXWINCE__
@@ -1086,51 +1111,55 @@ bool wxGenericFileDialog::Create( wxWindow *parent,
     style2 |= wxSUNKEN_BORDER;
 #endif
 
-    wxSize list_size(500,240);
-    if (is_pda) list_size = wxSize(50,80);
-
     m_list = new wxFileCtrl( this, ID_LIST_CTRL,
                              wxEmptyString, ms_lastShowHidden,
-                             wxDefaultPosition, list_size,
+                             wxDefaultPosition, wxSize(540,200),
                              style2);
 
+    m_text = new wxTextCtrl(this, ID_TEXT, m_fileName,
+                            wxDefaultPosition, wxDefaultSize,
+                            wxTE_PROCESS_ENTER);
+    m_choice = new wxChoice(this, ID_CHOICE);
+
     if (is_pda)
     {
         // PDAs have a different screen layout
-        mainsizer->Add( m_list, 1, wxEXPAND|wxSHRINK | wxLEFT|wxRIGHT, 5 );
+        mainsizer->Add(m_list, wxSizerFlags(1).Expand().HorzBorder());
 
-        wxBoxSizer *textsizer = new wxBoxSizer( wxHORIZONTAL );
-        m_text = new wxTextCtrl( this, ID_TEXT, m_fileName, wxDefaultPosition, wxDefaultSize, wxPROCESS_ENTER );
-        textsizer->Add( m_text, 1, wxCENTER | wxALL, 5 );
-        mainsizer->Add( textsizer, 0, wxEXPAND );
+        wxBoxSizer *textsizer = new wxBoxSizer(wxHORIZONTAL);
+        textsizer->Add(m_text, wxSizerFlags(1).Centre().Border());
+        mainsizer->Add(textsizer, wxSizerFlags().Expand());
 
         m_check = NULL;
-        m_choice = new wxChoice( this, ID_CHOICE );
-        textsizer->Add( m_choice, 1, wxCENTER|wxALL, 5 );
+        textsizer->Add(m_choice, wxSizerFlags(1).Centre().Border());
 
-        buttonsizer = new wxBoxSizer( wxHORIZONTAL );
-        buttonsizer->Add( new wxButton( this, wxID_OK ), 0, wxCENTER | wxALL, 5 );
-        buttonsizer->Add( new wxButton( this, wxID_CANCEL ), 0, wxCENTER | wxALL, 5 );
-        mainsizer->Add( buttonsizer, 0, wxALIGN_RIGHT );
+        wxSizer *bsizer = CreateButtonSizer(wxOK | wxCANCEL);
+        if ( bsizer )
+            mainsizer->Add(bsizer, wxSizerFlags().Expand().Border());
     }
-    else
+    else // !is_pda
     {
-        mainsizer->Add( m_list, 1, wxEXPAND | wxLEFT|wxRIGHT, 10 );
+        mainsizer->Add(m_list, wxSizerFlags(1).Expand().DoubleHorzBorder());
 
-        wxBoxSizer *textsizer = new wxBoxSizer( wxHORIZONTAL );
-        m_text = new wxTextCtrl( this, ID_TEXT, m_fileName, wxDefaultPosition, wxDefaultSize, wxPROCESS_ENTER );
-        textsizer->Add( m_text, 1, wxCENTER | wxLEFT|wxRIGHT|wxTOP, 10 );
-        textsizer->Add( new wxButton( this, wxID_OK ), 0, wxCENTER | wxLEFT|wxRIGHT|wxTOP, 10 );
-        mainsizer->Add( textsizer, 0, wxEXPAND );
+        wxBoxSizer *textsizer = new wxBoxSizer(wxHORIZONTAL);
+        textsizer->Add(m_text, wxSizerFlags(1).Centre().
+                                        DoubleBorder(wxLEFT | wxRIGHT | wxTOP));
+        textsizer->Add(new wxButton(this, wxID_OK), wxSizerFlags().Centre().
+                                        DoubleBorder(wxLEFT | wxRIGHT | wxTOP));
+        mainsizer->Add(textsizer, wxSizerFlags().Expand());
 
-        wxBoxSizer *choicesizer = new wxBoxSizer( wxHORIZONTAL );
-        m_choice = new wxChoice( this, ID_CHOICE );
-        choicesizer->Add( m_choice, 1, wxCENTER|wxALL, 10 );
-        m_check = new wxCheckBox( this, ID_CHECK, _("Show hidden files") );
-        m_check->SetValue( ms_lastShowHidden );
-        choicesizer->Add( m_check, 0, wxCENTER|wxALL, 10 );
-        choicesizer->Add( new wxButton( this, wxID_CANCEL ), 0, wxCENTER | wxALL, 10 );
-        mainsizer->Add( choicesizer, 0, wxEXPAND );
+        wxSizerFlags flagsCentre;
+        flagsCentre.Centre().DoubleBorder();
+
+        wxBoxSizer *choicesizer = new wxBoxSizer(wxHORIZONTAL);
+        choicesizer->Add(m_choice, wxSizerFlags(flagsCentre).Proportion(1));
+
+        m_check = new wxCheckBox(this, ID_CHECK, _("Show &hidden files"));
+        m_check->SetValue(ms_lastShowHidden);
+
+        choicesizer->Add(m_check, flagsCentre);
+        choicesizer->Add(new wxButton(this, wxID_CANCEL), flagsCentre);
+        mainsizer->Add(choicesizer, wxSizerFlags().Expand());
     }
 
     SetWildcard(wildCard);
@@ -1142,10 +1171,10 @@ bool wxGenericFileDialog::Create( wxWindow *parent,
     {
         mainsizer->Fit( this );
         mainsizer->SetSizeHints( this );
-    
+
         Centre( wxBOTH );
     }
-    
+
     m_text->SetFocus();
 
     ignoreChanges = false;
@@ -1269,9 +1298,7 @@ void wxGenericFileDialog::OnActivated( wxListEvent &event )
 
 void wxGenericFileDialog::OnTextEnter( wxCommandEvent &WXUNUSED(event) )
 {
-    wxCommandEvent cevent(wxEVT_COMMAND_BUTTON_CLICKED, wxID_OK);
-    cevent.SetEventObject( this );
-    GetEventHandler()->ProcessEvent( cevent );
+    HandleAction( m_text->GetValue() );
 }
 
 void wxGenericFileDialog::OnTextChange( wxCommandEvent &WXUNUSED(event) )
@@ -1307,13 +1334,21 @@ void wxGenericFileDialog::OnSelected( wxListEvent &event )
     // No double-click on most WinCE devices, so do action immediately.
     HandleAction( filename );
 #else
-    if (filename == wxT("..")) return;
+    if (filename == wxT(".."))
+    {
+        inSelected = false;
+        return;
+    }
 
     wxString dir = m_list->GetDir();
     if (!IsTopMostDir(dir))
         dir += wxFILE_SEP_PATH;
     dir += filename;
-    if (wxDirExists(dir)) return;
+    if (wxDirExists(dir))
+    {
+        inSelected = false;
+        return;
+    }
 
     ignoreChanges = true;
     m_text->SetValue( filename );
@@ -1328,10 +1363,17 @@ void wxGenericFileDialog::HandleAction( const wxString &fn )
         return;
 
     wxString filename( fn );
-    wxString dir = m_list->GetDir();
-    if (filename.empty()) return;
+    if (filename.empty())
+    {
+#ifdef __WXWINCE__
+        EndModal(wxID_CANCEL);
+#endif
+        return;
+    }
     if (filename == wxT(".")) return;
 
+    wxString dir = m_list->GetDir();
+
     // "some/place/" means they want to chdir not try to load "place"
     bool want_dir = filename.Last() == wxFILE_SEP_PATH;
     if (want_dir)
@@ -1364,7 +1406,7 @@ void wxGenericFileDialog::HandleAction( const wxString &fn )
     }
 #endif // __UNIX__
 
-    if (!(m_dialogStyle & wxSAVE))
+    if (!HasFdFlag(wxFD_SAVE))
     {
         if ((filename.Find(wxT('*')) != wxNOT_FOUND) ||
             (filename.Find(wxT('?')) != wxNOT_FOUND))
@@ -1409,14 +1451,13 @@ void wxGenericFileDialog::HandleAction( const wxString &fn )
     // VZ: the logic of testing for !wxFileExists() only for the open file
     //     dialog is not entirely clear to me, why don't we allow saving to a
     //     file without extension as well?
-    if ( !(m_dialogStyle & wxOPEN) || !wxFileExists(filename) )
+    if ( !HasFdFlag(wxFD_OPEN) || !wxFileExists(filename) )
     {
         filename = AppendExtension(filename, m_filterExtension);
     }
 
     // check that the file [doesn't] exist if necessary
-    if ( (m_dialogStyle & wxSAVE) &&
-            (m_dialogStyle & wxOVERWRITE_PROMPT) &&
+    if ( HasFdFlag(wxFD_SAVE) && HasFdFlag(wxFD_OVERWRITE_PROMPT) &&
                 wxFileExists( filename ) )
     {
         wxString msg;
@@ -1425,8 +1466,7 @@ void wxGenericFileDialog::HandleAction( const wxString &fn )
         if (wxMessageBox(msg, _("Confirm"), wxYES_NO) != wxYES)
             return;
     }
-    else if ( (m_dialogStyle & wxOPEN) &&
-                (m_dialogStyle & wxFILE_MUST_EXIST) &&
+    else if ( HasFdFlag(wxFD_OPEN) && HasFdFlag(wxFD_FILE_MUST_EXIST) &&
                     !wxFileExists(filename) )
     {
         wxMessageBox(_("Please choose an existing file."), _("Error"),
@@ -1436,7 +1476,7 @@ void wxGenericFileDialog::HandleAction( const wxString &fn )
     SetPath( filename );
 
     // change to the directory where the user went if asked
-    if ( m_dialogStyle & wxCHANGE_DIR )
+    if ( HasFdFlag(wxFD_CHANGE_DIR) )
     {
         wxString cwd;
         wxSplitPath(filename, &cwd, NULL, NULL);
@@ -1447,8 +1487,7 @@ void wxGenericFileDialog::HandleAction( const wxString &fn )
         }
     }
 
-    wxCommandEvent event;
-    wxDialog::OnOK(event);
+    EndModal(wxID_OK);
 }
 
 void wxGenericFileDialog::OnListOk( wxCommandEvent &WXUNUSED(event) )
@@ -1592,11 +1631,10 @@ void wxGenericFileDialog::UpdateControls()
 #endif // defined(__DOS__) || defined(__WINDOWS__) || defined(__OS2__)
 }
 
-#ifdef USE_GENERIC_FILEDIALOG
+#ifdef wxUSE_GENERIC_FILEDIALOG
 
-IMPLEMENT_DYNAMIC_CLASS(wxFileDialog, wxGenericFileDialog);
+IMPLEMENT_DYNAMIC_CLASS(wxFileDialog, wxGenericFileDialog)
 
-#endif // USE_GENERIC_FILEDIALOG
+#endif // wxUSE_GENERIC_FILEDIALOG
 
 #endif // wxUSE_FILEDLG
-
