@@ -1,10 +1,10 @@
 /////////////////////////////////////////////////////////////////////////////
-// Name:        msw/radiobox.cpp
+// Name:        src/msw/radiobox.cpp
 // Purpose:     wxRadioBox implementation
 // Author:      Julian Smart
 // Modified by:
 // Created:     04/01/98
-// RCS-ID:      $Id: radiobox.cpp,v 1.122.2.2 2006/01/18 16:16:21 JS Exp $
+// RCS-ID:      $Id: radiobox.cpp,v 1.142 2006/10/31 16:37:52 VZ Exp $
 // Copyright:   (c) Julian Smart
 // Licence:     wxWindows licence
 /////////////////////////////////////////////////////////////////////////////
@@ -17,10 +17,6 @@
 // headers
 // ---------------------------------------------------------------------------
 
-#if defined(__GNUG__) && !defined(NO_GCC_PRAGMA)
-    #pragma implementation "radiobox.h"
-#endif
-
 // For compilers that support precompilation, includes "wx.h".
 #include "wx/wxprec.h"
 
@@ -30,10 +26,11 @@
 
 #if wxUSE_RADIOBOX
 
+#include "wx/radiobox.h"
+
 #ifndef WX_PRECOMP
     #include "wx/bitmap.h"
     #include "wx/brush.h"
-    #include "wx/radiobox.h"
     #include "wx/settings.h"
     #include "wx/log.h"
 #endif
@@ -41,9 +38,6 @@
 #include "wx/msw/subwin.h"
 
 #if wxUSE_TOOLTIPS
-    #if !defined(__GNUWIN32_OLD__) || defined(__CYGWIN10__)
-        #include <commctrl.h>
-    #endif
     #include "wx/tooltip.h"
 #endif // wxUSE_TOOLTIPS
 
@@ -135,7 +129,6 @@ void wxRadioBox::Init()
 {
     m_selectedButton = wxNOT_FOUND;
     m_radioButtons = NULL;
-    m_majorDim = 0;
     m_radioWidth = NULL;
     m_radioHeight = NULL;
 }
@@ -152,9 +145,6 @@ bool wxRadioBox::Create(wxWindow *parent,
                         const wxValidator& val,
                         const wxString& name)
 {
-    // initialize members
-    m_majorDim = majorDim == 0 ? n : majorDim;
-
     // common initialization
     if ( !wxStaticBox::Create(parent, id, title, pos, size, style, name) )
         return false;
@@ -216,11 +206,12 @@ bool wxRadioBox::Create(wxWindow *parent,
     SetWindowPos(GetHwnd(), HWND_BOTTOM, 0, 0, 0, 0, SWP_NOMOVE|SWP_NOSIZE);
 #endif
 
+    SetMajorDim(majorDim == 0 ? n : majorDim, style);
     SetSelection(0);
     SetSize(pos.x, pos.y, size.x, size.y);
 
     // Now that we have items determine what is the best size and set it.
-    SetBestSize(size);
+    SetInitialSize(size);
 
     return true;
 }
@@ -276,8 +267,8 @@ bool wxRadioBox::MSWCommand(WXUINT cmd, WXWORD id)
 
         int selectedButton = wxNOT_FOUND;
 
-        int count = GetCount();
-        for ( int i = 0; i < count; i++ )
+        const unsigned int count = GetCount();
+        for ( unsigned int i = 0; i < count; i++ )
         {
             if ( id == wxGetWindowId((*m_radioButtons)[i]) )
             {
@@ -321,7 +312,7 @@ void wxRadioBox::SendNotificationEvent()
 {
     wxCommandEvent event(wxEVT_COMMAND_RADIOBOX_SELECTED, m_windowId);
     event.SetInt( m_selectedButton );
-    event.SetString( GetString(m_selectedButton) );
+    event.SetString(GetString(m_selectedButton));
     event.SetEventObject( this );
     ProcessCommand(event);
 }
@@ -330,38 +321,12 @@ void wxRadioBox::SendNotificationEvent()
 // simple accessors
 // ----------------------------------------------------------------------------
 
-int wxRadioBox::GetCount() const
+unsigned int wxRadioBox::GetCount() const
 {
-    return m_radioButtons->GetCount();
+    return m_radioButtons ? m_radioButtons->GetCount() : 0u;
 }
 
-// returns the number of rows
-int wxRadioBox::GetNumVer() const
-{
-    if ( m_windowStyle & wxRA_SPECIFY_ROWS )
-    {
-        return m_majorDim;
-    }
-    else
-    {
-        return (GetCount() + m_majorDim - 1)/m_majorDim;
-    }
-}
-
-// returns the number of columns
-int wxRadioBox::GetNumHor() const
-{
-    if ( m_windowStyle & wxRA_SPECIFY_ROWS )
-    {
-        return (GetCount() + m_majorDim - 1)/m_majorDim;
-    }
-    else
-    {
-        return m_majorDim;
-    }
-}
-
-void wxRadioBox::SetString(int item, const wxString& label)
+void wxRadioBox::SetString(unsigned int item, const wxString& label)
 {
     wxCHECK_RET( IsValid(item), wxT("invalid radiobox index") );
 
@@ -388,7 +353,7 @@ void wxRadioBox::SetSelection(int N)
 }
 
 // Find string for position
-wxString wxRadioBox::GetString(int item) const
+wxString wxRadioBox::GetString(unsigned int item) const
 {
     wxCHECK_MSG( IsValid(item), wxEmptyString,
                  wxT("invalid radiobox index") );
@@ -407,30 +372,72 @@ void wxRadioBox::SetFocus()
 }
 
 // Enable a specific button
-bool wxRadioBox::Enable(int item, bool enable)
+bool wxRadioBox::Enable(unsigned int item, bool enable)
 {
     wxCHECK_MSG( IsValid(item), false,
                  wxT("invalid item in wxRadioBox::Enable()") );
 
     BOOL ret = ::EnableWindow((*m_radioButtons)[item], enable);
 
-    return (ret == 0) == enable;
+    return (ret == 0) != enable;
+}
+
+bool wxRadioBox::IsItemEnabled(unsigned int item) const
+{
+    wxCHECK_MSG( IsValid(item), false,
+                 wxT("invalid item in wxRadioBox::IsItemEnabled()") );
+
+    return ::IsWindowEnabled((*m_radioButtons)[item]) != 0;
 }
 
 // Show a specific button
-bool wxRadioBox::Show(int item, bool show)
+bool wxRadioBox::Show(unsigned int item, bool show)
 {
     wxCHECK_MSG( IsValid(item), false,
                  wxT("invalid item in wxRadioBox::Show()") );
 
     BOOL ret = ::ShowWindow((*m_radioButtons)[item], show ? SW_SHOW : SW_HIDE);
 
-    bool changed = (ret != 0) == show;
-    if( changed )
+    bool changed = (ret != 0) != show;
+    if ( changed )
+    {
         InvalidateBestSize();
+    }
+
     return changed;
 }
 
+bool wxRadioBox::IsItemShown(unsigned int item) const
+{
+    wxCHECK_MSG( IsValid(item), false,
+                 wxT("invalid item in wxRadioBox::IsItemShown()") );
+
+    // don't use IsWindowVisible() here because it would return false if the
+    // radiobox itself is hidden while we want to only return false if this
+    // button specifically is hidden
+    return (::GetWindowLong((*m_radioButtons)[item],
+                            GWL_STYLE) & WS_VISIBLE) != 0;
+}
+
+#if wxUSE_TOOLTIPS
+
+bool wxRadioBox::HasToolTips() const
+{
+    return wxStaticBox::HasToolTips() || wxRadioBoxBase::HasItemToolTips();
+}
+
+void wxRadioBox::DoSetItemToolTip(unsigned int item, wxToolTip *tooltip)
+{
+    // we have already checked for the item to be valid in wxRadioBoxBase
+    const HWND hwndRbtn = (*m_radioButtons)[item];
+    if ( tooltip != NULL )
+        tooltip->Add(hwndRbtn);
+    else // unset the tooltip
+        wxToolTip::Remove(hwndRbtn);
+}
+
+#endif // wxUSE_TOOLTIPS
+
 WX_FORWARD_STD_METHODS_TO_SUBWINDOWS(wxRadioBox, wxStaticBox, m_radioButtons)
 
 // ----------------------------------------------------------------------------
@@ -442,13 +449,13 @@ wxSize wxRadioBox::GetMaxButtonSize() const
     // calculate the max button size
     int widthMax = 0,
         heightMax = 0;
-    const int count = GetCount();
-    for ( int i = 0 ; i < count; i++ )
+    const unsigned int count = GetCount();
+    for ( unsigned int i = 0 ; i < count; i++ )
     {
         int width, height;
         if ( m_radioWidth[i] < 0 )
         {
-            GetTextExtent(wxStripMenuCodes(wxGetWindowText((*m_radioButtons)[i])), &width, &height);
+            GetTextExtent(wxGetWindowText((*m_radioButtons)[i]), &width, &height);
 
             // adjust the size to take into account the radio box itself
             // FIXME this is totally bogus!
@@ -479,8 +486,8 @@ wxSize wxRadioBox::GetTotalButtonSize(const wxSize& sizeBtn) const
 
     int extraHeight = cy1;
 
-    int height = GetNumVer() * sizeBtn.y + cy1/2 + extraHeight;
-    int width  = GetNumHor() * (sizeBtn.x + cx1) + cx1;
+    int height = GetRowCount() * sizeBtn.y + cy1/2 + extraHeight;
+    int width  = GetColumnCount() * (sizeBtn.x + cx1) + cx1;
 
     // Add extra space under the label, if it exists.
     if (!wxControl::GetLabel().empty())
@@ -488,7 +495,7 @@ wxSize wxRadioBox::GetTotalButtonSize(const wxSize& sizeBtn) const
 
     // and also wide enough for its label
     int widthLabel;
-    GetTextExtent(wxStripMenuCodes(GetTitle()), &widthLabel, NULL);
+    GetTextExtent(GetLabelText(), &widthLabel, NULL);
     widthLabel += RADIO_SIZE; // FIXME this is bogus too
     if ( widthLabel > width )
         width = widthLabel;
@@ -498,6 +505,13 @@ wxSize wxRadioBox::GetTotalButtonSize(const wxSize& sizeBtn) const
 
 wxSize wxRadioBox::DoGetBestSize() const
 {
+    if ( !m_radioButtons )
+    {
+        // if we're not fully initialized yet, we can't meaningfully compute
+        // our best size, we'll do it later
+        return wxSize(1, 1);
+    }
+
     wxSize best = GetTotalButtonSize(GetMaxButtonSize());
     CacheBestSize(best);
     return best;
@@ -561,9 +575,9 @@ void wxRadioBox::DoSetSize(int x, int y, int width, int height, int sizeFlags)
     // to the right border of radiobox and thus can be wider than this.
 
     // Also, remember that wxRA_SPECIFY_COLS means that we arrange buttons in
-    // left to right order and m_majorDim is the number of columns while
+    // left to right order and GetMajorDim() is the number of columns while
     // wxRA_SPECIFY_ROWS means that the buttons are arranged top to bottom and
-    // m_majorDim is the number of rows.
+    // GetMajorDim() is the number of rows.
 
     x_offset += cx1;
     y_offset += cy1;
@@ -575,8 +589,8 @@ void wxRadioBox::DoSetSize(int x, int y, int width, int height, int sizeFlags)
     int startX = x_offset;
     int startY = y_offset;
 
-    const int count = GetCount();
-    for ( int i = 0; i < count; i++ )
+    const unsigned int count = GetCount();
+    for (unsigned int i = 0; i < count; i++)
     {
         // the last button in the row may be wider than the other ones as the
         // radiobox may be wider than the sum of the button widths (as it
@@ -586,17 +600,17 @@ void wxRadioBox::DoSetSize(int x, int y, int width, int height, int sizeFlags)
         {
             // item is the last in its row if it is a multiple of the number of
             // columns or if it is just the last item
-            int n = i + 1;
-            isLastInTheRow = ((n % m_majorDim) == 0) || (n == count);
+            unsigned int n = i + 1;
+            isLastInTheRow = ((n % GetMajorDim()) == 0) || (n == count);
         }
         else // wxRA_SPECIFY_ROWS
         {
             // item is the last in the row if it is in the last columns
-            isLastInTheRow = i >= (count/m_majorDim)*m_majorDim;
+            isLastInTheRow = i >= (count/GetMajorDim())*GetMajorDim();
         }
 
         // is this the start of new row/column?
-        if ( i && (i % m_majorDim == 0) )
+        if ( i && (i % GetMajorDim() == 0) )
         {
             if ( m_windowStyle & wxRA_SPECIFY_ROWS )
             {
@@ -649,6 +663,23 @@ void wxRadioBox::DoSetSize(int x, int y, int width, int height, int sizeFlags)
     }
 }
 
+int wxRadioBox::GetItemFromPoint(const wxPoint& pt) const
+{
+    const unsigned int count = GetCount();
+    for ( unsigned int i = 0; i < count; i++ )
+    {
+        RECT rect = wxGetWindowRect((*m_radioButtons)[i]);
+
+        if ( rect.left <= pt.x && pt.x < rect.right &&
+                rect.top  <= pt.y && pt.y < rect.bottom )
+        {
+            return i;
+        }
+    }
+
+    return wxNOT_FOUND;
+}
+
 // ----------------------------------------------------------------------------
 // radio box drawing
 // ----------------------------------------------------------------------------
@@ -661,9 +692,13 @@ WXHRGN wxRadioBox::MSWGetRegionWithoutChildren()
     ::GetWindowRect(GetHwnd(), &rc);
     HRGN hrgn = ::CreateRectRgn(rc.left, rc.top, rc.right + 1, rc.bottom + 1);
 
-    const size_t count = GetCount();
-    for ( size_t i = 0; i < count; ++i )
+    const unsigned int count = GetCount();
+    for ( unsigned int i = 0; i < count; ++i )
     {
+        // don't clip out hidden children
+        if ( !IsItemShown(i) )
+            continue;
+
         ::GetWindowRect((*m_radioButtons)[i], &rc);
         AutoHRGN hrgnchild(::CreateRectRgnIndirect(&rc));
         ::CombineRgn(hrgn, hrgn, hrgnchild, RGN_DIFF);
@@ -672,12 +707,6 @@ WXHRGN wxRadioBox::MSWGetRegionWithoutChildren()
     return (WXHRGN)hrgn;
 }
 
-WXLRESULT
-wxRadioBox::MSWWindowProc(WXUINT nMsg, WXWPARAM wParam, WXLPARAM lParam)
-{
-    return wxStaticBox::MSWWindowProc(nMsg, wParam, lParam);
-}
-
 #endif // __WXWINCE__
 
 // ---------------------------------------------------------------------------
@@ -702,32 +731,6 @@ LRESULT APIENTRY _EXPORT wxRadioBtnWndProc(HWND hwnd,
                 return lDlgCode | DLGC_WANTARROWS;
             }
 
-#if wxUSE_TOOLTIPS
-        case WM_NOTIFY:
-            {
-                NMHDR* hdr = (NMHDR *)lParam;
-                if ( hdr->code == TTN_NEEDTEXT )
-                {
-                    wxRadioBox *
-                        radiobox = (wxRadioBox *)wxGetWindowUserData(hwnd);
-
-                    wxCHECK_MSG( radiobox, 0,
-                                 wxT("radio button without radio box?") );
-
-                    wxToolTip *tooltip = radiobox->GetToolTip();
-                    if ( tooltip )
-                    {
-                        TOOLTIPTEXT *ttt = (TOOLTIPTEXT *)lParam;
-                        ttt->lpszText = (wxChar *)tooltip->GetTip().c_str();
-                    }
-
-                    // processed
-                    return 0;
-                }
-            }
-            break;
-#endif // wxUSE_TOOLTIPS
-
         case WM_KEYDOWN:
             {
                 wxRadioBox *radiobox = (wxRadioBox *)wxGetWindowUserData(hwnd);
@@ -852,4 +855,3 @@ LRESULT APIENTRY _EXPORT wxRadioBtnWndProc(HWND hwnd,
 }
 
 #endif // wxUSE_RADIOBOX
-
