@@ -1,10 +1,10 @@
 /////////////////////////////////////////////////////////////////////////////
-// Name:        msw/tbar95.cpp
+// Name:        src/msw/tbar95.cpp
 // Purpose:     wxToolBar
 // Author:      Julian Smart
 // Modified by:
 // Created:     04/01/98
-// RCS-ID:      $Id: tbar95.cpp,v 1.157.2.8 2006/04/01 18:16:17 JS Exp $
+// RCS-ID:      $Id: tbar95.cpp,v 1.192 2007/01/12 18:27:27 RD Exp $
 // Copyright:   (c) Julian Smart
 // Licence:     wxWindows licence
 /////////////////////////////////////////////////////////////////////////////
@@ -17,10 +17,6 @@
 // headers
 // ----------------------------------------------------------------------------
 
-#if defined(__GNUG__) && !defined(NO_GCC_PRAGMA)
-    #pragma implementation "tbar95.h"
-#endif
-
 // For compilers that support precompilation, includes "wx.h".
 #include "wx/wxprec.h"
 
@@ -28,22 +24,25 @@
     #pragma hdrstop
 #endif
 
+#if wxUSE_TOOLBAR && wxUSE_TOOLBAR_NATIVE && !defined(__SMARTPHONE__)
+
+#include "wx/toolbar.h"
+
 #ifndef WX_PRECOMP
+    #include "wx/msw/wrapcctl.h" // include <commctrl.h> "properly"
+    #include "wx/dynarray.h"
     #include "wx/frame.h"
     #include "wx/log.h"
     #include "wx/intl.h"
-    #include "wx/dynarray.h"
     #include "wx/settings.h"
     #include "wx/bitmap.h"
     #include "wx/dcmemory.h"
     #include "wx/control.h"
+    #include "wx/app.h"         // for GetComCtl32Version
+    #include "wx/image.h"
 #endif
 
-#if wxUSE_TOOLBAR && wxUSE_TOOLBAR_NATIVE && !defined(__SMARTPHONE__)
-
-#include "wx/toolbar.h"
 #include "wx/sysopt.h"
-#include "wx/image.h"
 
 #include "wx/msw/private.h"
 
@@ -51,10 +50,14 @@
 #include "wx/msw/uxtheme.h"
 #endif
 
-// include <commctrl.h> "properly"
-#include "wx/msw/wrapcctl.h"
-
-#include "wx/app.h"         // for GetComCtl32Version
+// this define controls whether the code for button colours remapping (only
+// useful for 16 or 256 colour images) is active at all, it's always turned off
+// for CE where it doesn't compile (and is probably not needed anyhow) and may
+// also be turned off for other systems if you always use 24bpp images and so
+// never need it
+#ifndef __WXWINCE__
+    #define wxREMAP_BUTTON_COLOURS
+#endif // !__WXWINCE__
 
 // ----------------------------------------------------------------------------
 // constants
@@ -177,7 +180,6 @@ private:
     DECLARE_NO_COPY_CLASS(wxToolBarTool)
 };
 
-
 // ============================================================================
 // implementation
 // ============================================================================
@@ -232,28 +234,23 @@ bool wxToolBar::Create(wxWindow *parent,
     if ( !CreateControl(parent, id, pos, size, style, wxDefaultValidator, name) )
         return false;
 
+    FixupStyle();
+
     // MSW-specific initialisation
     if ( !MSWCreateToolbar(pos, size) )
         return false;
 
     wxSetCCUnicodeFormat(GetHwnd());
 
-    // set up the colors and fonts
-    SetBackgroundColour(wxSystemSettings::GetColour(wxSYS_COLOUR_BTNFACE));
-    SetFont(wxSystemSettings::GetFont(wxSYS_DEFAULT_GUI_FONT));
-
     // workaround for flat toolbar on Windows XP classic style: we have to set
-    // the style after creating the control, doing it at creation time doesn't
-    // work
+    // the style after creating the control; doing it at creation time doesn't work
 #if wxUSE_UXTHEME
     if ( style & wxTB_FLAT )
     {
-        LRESULT style = ::SendMessage(GetHwnd(), TB_GETSTYLE, 0, 0L);
+        LRESULT style = GetMSWToolbarStyle();
 
         if ( !(style & TBSTYLE_FLAT) )
-        {
             ::SendMessage(GetHwnd(), TB_SETSTYLE, 0, style | TBSTYLE_FLAT);
-        }
     }
 #endif // wxUSE_UXTHEME
 
@@ -304,8 +301,8 @@ void wxToolBar::Recreate()
             ::SetParent(GetHwndOf(win), GetHwnd());
     }
 
-    // only destroy the old toolbar now -- after all the children had been
-    // reparented
+    // only destroy the old toolbar now --
+    // after all the children had been reparented
     ::DestroyWindow(hwndOld);
 
     // it is for the old bitmap control and can't be used with the new one
@@ -330,14 +327,10 @@ wxToolBar::~wxToolBar()
     // is not - otherwise toolbar leaves a hole in the place it used to occupy
     wxFrame *frame = wxDynamicCast(GetParent(), wxFrame);
     if ( frame && !frame->IsBeingDeleted() )
-    {
         frame->SendSizeEvent();
-    }
 
     if ( m_hBitmap )
-    {
         ::DeleteObject((HBITMAP) m_hBitmap);
-    }
 
     delete m_disabledImgList;
 }
@@ -356,7 +349,7 @@ wxSize wxToolBar::DoGetBestSize() const
         sizeBest.x *= GetToolsCount();
 
         // reverse horz and vertical components if necessary
-        if ( HasFlag(wxTB_VERTICAL) )
+        if ( IsVertical() )
         {
             int t = sizeBest.x;
             sizeBest.x = sizeBest.y;
@@ -370,6 +363,7 @@ wxSize wxToolBar::DoGetBestSize() const
     }
 
     CacheBestSize(sizeBest);
+
     return sizeBest;
 }
 
@@ -382,28 +376,23 @@ WXDWORD wxToolBar::MSWGetStyle(long style, WXDWORD *exstyle) const
                         (style & ~wxBORDER_MASK) | wxBORDER_NONE, exstyle
                       );
 
-    // always include this one, it never hurts and setting it later only if we
-    // do have tooltips wouldn't work
-    msStyle |= TBSTYLE_TOOLTIPS;
+    if ( !(style & wxTB_NO_TOOLTIPS) )
+        msStyle |= TBSTYLE_TOOLTIPS;
 
     if ( style & (wxTB_FLAT | wxTB_HORZ_LAYOUT) )
     {
         // static as it doesn't change during the program lifetime
-        static int s_verComCtl = wxApp::GetComCtl32Version();
+        static const int s_verComCtl = wxApp::GetComCtl32Version();
 
         // comctl32.dll 4.00 doesn't support the flat toolbars and using this
         // style with 6.00 (part of Windows XP) leads to the toolbar with
         // incorrect background colour - and not using it still results in the
         // correct (flat) toolbar, so don't use it there
         if ( s_verComCtl > 400 && s_verComCtl < 600 )
-        {
             msStyle |= TBSTYLE_FLAT | TBSTYLE_TRANSPARENT;
-        }
 
         if ( s_verComCtl >= 470 && style & wxTB_HORZ_LAYOUT )
-        {
             msStyle |= TBSTYLE_LIST;
-        }
     }
 
     if ( style & wxTB_NODIVIDER )
@@ -415,6 +404,12 @@ WXDWORD wxToolBar::MSWGetStyle(long style, WXDWORD *exstyle) const
     if ( style & wxTB_VERTICAL )
         msStyle |= CCS_VERT;
 
+    if( style & wxTB_BOTTOM )
+        msStyle |= CCS_BOTTOM;
+
+    if ( style & wxTB_RIGHT )
+        msStyle |= CCS_RIGHT;
+
     return msStyle;
 }
 
@@ -454,9 +449,7 @@ bool wxToolBar::DoDeleteTool(size_t pos, wxToolBarToolBase *tool)
         }
 
         if ( tool2->IsControl() )
-        {
             pos += ((wxToolBarTool *)tool2)->GetSeparatorsCount() - 1;
-        }
     }
 
     // now determine the number of buttons to delete and the area taken by them
@@ -507,14 +500,21 @@ bool wxToolBar::DoDeleteTool(size_t pos, wxToolBarToolBase *tool)
     }
 
     InvalidateBestSize();
+
     return true;
 }
 
 void wxToolBar::CreateDisabledImageList()
 {
+    if (m_disabledImgList != NULL)
+    {
+        delete m_disabledImgList;
+        m_disabledImgList = NULL;
+    }
+
     // as we can't use disabled image list with older versions of comctl32.dll,
     // don't even bother creating it
-    if ( wxTheApp->GetComCtl32Version() >= 470 )
+    if ( wxApp::GetComCtl32Version() >= 470 )
     {
         // search for the first disabled button img in the toolbar, if any
         for ( wxToolBarToolsList::compatibility_iterator
@@ -531,45 +531,42 @@ void wxToolBar::CreateDisabledImageList()
                                             bmpDisabled.GetMask() != NULL,
                                             GetToolsCount()
                                         );
-                return;
+                break;
             }
         }
 
         // we don't have any disabled bitmaps
     }
-
-    m_disabledImgList = NULL;
 }
 
 bool wxToolBar::Realize()
 {
     const size_t nTools = GetToolsCount();
     if ( nTools == 0 )
-    {
         // nothing to do
         return true;
-    }
 
-    const bool isVertical = HasFlag(wxTB_VERTICAL);
-
-    bool doRemap, doRemapBg, doTransparent;
-#ifdef __WXWINCE__
-    doRemapBg = false;
-    doRemap = false;
-    doTransparent = false;
-#else
-    if (wxSystemOptions::GetOptionInt(wxT("msw.remap")) == 2)
+#ifdef wxREMAP_BUTTON_COLOURS
+    // don't change the values of these constants, they can be set from the
+    // user code via wxSystemOptions
+    enum
     {
-        doRemapBg = doRemap = false;
-        doTransparent = true;
-    }
-    else
-    {   doRemap = !wxSystemOptions::HasOption(wxT("msw.remap"))
-            || wxSystemOptions::GetOptionInt(wxT("msw.remap")) == 1;
-        doRemapBg = !doRemap;
-        doTransparent = false;
-    }
-#endif
+        Remap_None = -1,
+        Remap_Bg,
+        Remap_Buttons,
+        Remap_TransparentBg
+    };
+
+    // the user-specified option overrides anything, but if it wasn't set, only
+    // remap the buttons on 8bpp displays as otherwise the bitmaps usually look
+    // much worse after remapping
+    static const wxChar *remapOption = wxT("msw.remap");
+    const int remapValue = wxSystemOptions::HasOption(remapOption)
+                                ? wxSystemOptions::GetOptionInt(remapOption)
+                                : wxDisplayDepth() <= 8 ? Remap_Buttons
+                                                        : Remap_None;
+
+#endif // wxREMAP_BUTTON_COLOURS
 
     // delete all old buttons, if any
     for ( size_t pos = 0; pos < m_nButtons; pos++ )
@@ -606,25 +603,31 @@ bool wxToolBar::Realize()
                                           wx_truncate_cast(wxCoord, nTools),
                       totalBitmapHeight = m_defaultHeight;
 
-        // Create a bitmap and copy all the tool bitmaps to it
+        // Create a bitmap and copy all the tool bitmaps into it
         wxMemoryDC dcAllButtons;
         wxBitmap bitmap(totalBitmapWidth, totalBitmapHeight);
         dcAllButtons.SelectObject(bitmap);
+
+#ifdef wxREMAP_BUTTON_COLOURS
+        if ( remapValue != Remap_TransparentBg )
+#endif // wxREMAP_BUTTON_COLOURS
+        {
+            // VZ: why do we hardcode grey colour for CE?
+            dcAllButtons.SetBackground(wxBrush(
 #ifdef __WXWINCE__
-        dcAllButtons.SetBackground(wxBrush(wxColour(192,192,192)));
-#else
-        if (doTransparent)
-            dcAllButtons.SetBackground(*wxTRANSPARENT_BRUSH);
-        else
-            dcAllButtons.SetBackground(wxBrush(GetBackgroundColour()));
-#endif
-        dcAllButtons.Clear();
+                                        wxColour(0xc0, 0xc0, 0xc0)
+#else // !__WXWINCE__
+                                        GetBackgroundColour()
+#endif // __WXWINCE__/!__WXWINCE__
+                                       ));
+            dcAllButtons.Clear();
+        }
 
         m_hBitmap = bitmap.GetHBITMAP();
         HBITMAP hBitmap = (HBITMAP)m_hBitmap;
 
-#ifndef __WXWINCE__
-        if (doRemapBg)
+#ifdef wxREMAP_BUTTON_COLOURS
+        if ( remapValue == Remap_Bg )
         {
             dcAllButtons.SelectObject(wxNullBitmap);
 
@@ -634,10 +637,8 @@ bool wxToolBar::Realize()
                 totalBitmapWidth, totalBitmapHeight);
 
             dcAllButtons.SelectObject(bitmap);
-
-
         }
-#endif // !__WXWINCE__
+#endif // wxREMAP_BUTTON_COLOURS
 
         // the button position
         wxCoord x = 0;
@@ -682,25 +683,33 @@ bool wxToolBar::Realize()
                         wxImage imgGreyed;
                         wxCreateGreyedImage(bmp.ConvertToImage(), imgGreyed);
 
-                        // we need to have light grey background colour for
-                        // MapBitmap() to work correctly
-                        for ( int y = 0; y < h; y++ )
+#ifdef wxREMAP_BUTTON_COLOURS
+                        if ( remapValue == Remap_Buttons )
                         {
-                            for ( int x = 0; x < w; x++ )
+                            // we need to have light grey background colour for
+                            // MapBitmap() to work correctly
+                            for ( int y = 0; y < h; y++ )
                             {
-                                if ( imgGreyed.IsTransparent(x, y) )
-                                    imgGreyed.SetRGB(x, y,
-                                                     wxLIGHT_GREY->Red(),
-                                                     wxLIGHT_GREY->Green(),
-                                                     wxLIGHT_GREY->Blue());
+                                for ( int x = 0; x < w; x++ )
+                                {
+                                    if ( imgGreyed.IsTransparent(x, y) )
+                                        imgGreyed.SetRGB(x, y,
+                                                         wxLIGHT_GREY->Red(),
+                                                         wxLIGHT_GREY->Green(),
+                                                         wxLIGHT_GREY->Blue());
+                                }
                             }
                         }
+#endif // wxREMAP_BUTTON_COLOURS
 
                         bmpDisabled = wxBitmap(imgGreyed);
                     }
 #endif // wxUSE_IMAGE
 
-                    MapBitmap(bmpDisabled.GetHBITMAP(), w, h);
+#ifdef wxREMAP_BUTTON_COLOURS
+                    if ( remapValue == Remap_Buttons )
+                        MapBitmap(bmpDisabled.GetHBITMAP(), w, h);
+#endif // wxREMAP_BUTTON_COLOURS
 
                     m_disabledImgList->Add(bmpDisabled);
                 }
@@ -718,14 +727,14 @@ bool wxToolBar::Realize()
         // don't delete this HBITMAP!
         bitmap.SetHBITMAP(0);
 
-        if (doRemap)
+#ifdef wxREMAP_BUTTON_COLOURS
+        if ( remapValue == Remap_Buttons )
         {
             // Map to system colours
             hBitmap = (HBITMAP)MapBitmap((WXHBITMAP) hBitmap,
-                totalBitmapWidth, totalBitmapHeight);
-
-
+                                         totalBitmapWidth, totalBitmapHeight);
         }
+#endif // wxREMAP_BUTTON_COLOURS
 
         bool addBitmap = true;
 
@@ -775,17 +784,21 @@ bool wxToolBar::Realize()
             }
         }
 
-        if ( m_disabledImgList )
+        // disable image lists are only supported in comctl32.dll 4.70+
+        if ( wxApp::GetComCtl32Version() >= 470 )
         {
+            HIMAGELIST hil = m_disabledImgList
+                                ? GetHimagelistOf(m_disabledImgList)
+                                : 0;
+
+            // notice that we set the image list even if don't have one right
+            // now as we could have it before and need to reset it in this case
             HIMAGELIST oldImageList = (HIMAGELIST)
-                ::SendMessage(GetHwnd(),
-                              TB_SETDISABLEDIMAGELIST,
-                              0,
-                              (LPARAM)GetHimagelistOf(m_disabledImgList));
+              ::SendMessage(GetHwnd(), TB_SETDISABLEDIMAGELIST, 0, (LPARAM)hil);
 
             // delete previous image list if any
             if ( oldImageList )
-                ::DeleteObject( oldImageList );
+                ::DeleteObject(oldImageList);
         }
     }
 
@@ -813,13 +826,12 @@ bool wxToolBar::Realize()
 
         // don't add separators to the vertical toolbar with old comctl32.dll
         // versions as they didn't handle this properly
-        if ( isVertical && tool->IsSeparator() &&
+        if ( IsVertical() && tool->IsSeparator() &&
                 wxApp::GetComCtl32Version() <= 472 )
         {
             continue;
         }
 
-
         TBBUTTON& button = buttons[i];
 
         wxZeroMemory(button);
@@ -844,9 +856,7 @@ bool wxToolBar::Realize()
                 {
                     const wxString& label = tool->GetLabel();
                     if ( !label.empty() )
-                    {
                         button.iString = (int)label.c_str();
-                    }
                 }
 
                 button.idCommand = tool->GetId();
@@ -885,9 +895,8 @@ bool wxToolBar::Realize()
                                     break;
 
                                 if ( tool->Toggle(false) )
-                                {
                                     DoToggleTool(tool, false);
-                                }
+
                                 prevButton.fsState = TBSTATE_ENABLED;
                                 nodePrev = nodePrev->GetPrevious();
                                 prevIndex--;
@@ -901,12 +910,14 @@ bool wxToolBar::Realize()
                         button.fsStyle = TBSTYLE_CHECK;
                         break;
 
-                    default:
-                        wxFAIL_MSG( _T("unexpected toolbar button kind") );
-                        // fall through
-
                     case wxITEM_NORMAL:
                         button.fsStyle = TBSTYLE_BUTTON;
+                        break;
+
+                    default:
+                        wxFAIL_MSG( _T("unexpected toolbar button kind") );
+                        button.fsStyle = TBSTYLE_BUTTON;
+                        break;
                 }
 
                 bitmapId++;
@@ -939,7 +950,7 @@ bool wxToolBar::Realize()
         // get the items size for all items but for the horizontal ones we
         // don't need to deal with the non controls
         bool isControl = tool->IsControl();
-        if ( !isControl && !isVertical )
+        if ( !isControl && !IsVertical() )
             continue;
 
         // note that we use TB_GETITEMRECT and not TB_GETRECT because the
@@ -960,7 +971,6 @@ bool wxToolBar::Realize()
         }
 
         wxControl *control = tool->GetControl();
-
         wxSize size = control->GetSize();
 
         // the position of the leftmost controls corner
@@ -1031,12 +1041,12 @@ bool wxToolBar::Realize()
         }
 
         int top;
-        if ( isVertical )
+        if ( IsVertical() )
         {
             left = 0;
             top = y;
 
-            y += height + 2*GetMargins().y;
+            y += height + 2 * GetMargins().y;
         }
         else // horizontal toolbar
         {
@@ -1053,21 +1063,17 @@ bool wxToolBar::Realize()
     // separators which we added just for aligning the controls
     m_nButtons = index;
 
-    if ( !isVertical )
+    if ( !IsVertical() )
     {
         if ( m_maxRows == 0 )
-        {
             // if not set yet, only one row
             SetRows(1);
-        }
     }
     else if ( m_nButtons > 0 ) // vertical non empty toolbar
     {
-        if ( m_maxRows == 0 )
-        {
-            // if not set yet, have one column
-            SetRows(m_nButtons);
-        }
+        // if not set yet, have one column
+        m_maxRows = 1;
+        SetRows(m_nButtons);        
     }
 
     InvalidateBestSize();
@@ -1119,33 +1125,34 @@ bool wxToolBar::MSWOnNotify(int WXUNUSED(idCtrl),
                             WXLPARAM lParam,
                             WXLPARAM *WXUNUSED(result))
 {
+    if( !HasFlag(wxTB_NO_TOOLTIPS) )
+    {
 #if wxUSE_TOOLTIPS
-    // First check if this applies to us
-    NMHDR *hdr = (NMHDR *)lParam;
+        // First check if this applies to us
+        NMHDR *hdr = (NMHDR *)lParam;
 
-    // the tooltips control created by the toolbar is sometimes Unicode, even
-    // in an ANSI application - this seems to be a bug in comctl32.dll v5
-    UINT code = hdr->code;
-    if ( (code != (UINT) TTN_NEEDTEXTA) && (code != (UINT) TTN_NEEDTEXTW) )
-        return false;
+        // the tooltips control created by the toolbar is sometimes Unicode, even
+        // in an ANSI application - this seems to be a bug in comctl32.dll v5
+        UINT code = hdr->code;
+        if ( (code != (UINT) TTN_NEEDTEXTA) && (code != (UINT) TTN_NEEDTEXTW) )
+            return false;
 
-    HWND toolTipWnd = (HWND)::SendMessage((HWND)GetHWND(), TB_GETTOOLTIPS, 0, 0);
-    if ( toolTipWnd != hdr->hwndFrom )
-        return false;
+        HWND toolTipWnd = (HWND)::SendMessage(GetHwnd(), TB_GETTOOLTIPS, 0, 0);
+        if ( toolTipWnd != hdr->hwndFrom )
+            return false;
 
-    LPTOOLTIPTEXT ttText = (LPTOOLTIPTEXT)lParam;
-    int id = (int)ttText->hdr.idFrom;
+        LPTOOLTIPTEXT ttText = (LPTOOLTIPTEXT)lParam;
+        int id = (int)ttText->hdr.idFrom;
 
-    wxToolBarToolBase *tool = FindById(id);
-    if ( !tool )
-        return false;
-
-    return HandleTooltipNotify(code, lParam, tool->GetShortHelp());
+        wxToolBarToolBase *tool = FindById(id);
+        if ( tool )
+            return HandleTooltipNotify(code, lParam, tool->GetShortHelp());
 #else
-    wxUnusedVar(lParam);
+        wxUnusedVar(lParam);
+#endif
+    }
 
     return false;
-#endif
 }
 
 // ----------------------------------------------------------------------------
@@ -1229,13 +1236,12 @@ wxToolBarToolBase *wxToolBar::FindToolForPosition(wxCoord x, wxCoord y) const
     pt.x = x;
     pt.y = y;
     int index = (int)::SendMessage(GetHwnd(), TB_HITTEST, 0, (LPARAM)&pt);
+
     // MBN: when the point ( x, y ) is close to the toolbar border
     //      TB_HITTEST returns m_nButtons ( not -1 )
     if ( index < 0 || (size_t)index >= m_nButtons )
-    {
         // it's a separator or there is no tool at all there
         return (wxToolBarToolBase *)NULL;
-    }
 
     // when TB_SETBUTTONINFO is available (both during compile- and run-time),
     // we don't use the dummy separators hack
@@ -1274,6 +1280,12 @@ void wxToolBar::UpdateSize()
 // toolbar styles
 // ---------------------------------------------------------------------------
 
+// get the TBSTYLE of the given toolbar window
+long wxToolBar::GetMSWToolbarStyle() const
+{
+    return ::SendMessage(GetHwnd(), TB_GETSTYLE, 0, 0L);
+}
+
 void wxToolBar::SetWindowStyleFlag(long style)
 {
     // the style bits whose changes force us to recreate the toolbar
@@ -1354,7 +1366,7 @@ void wxToolBar::OnMouseEvent(wxMouseEvent& event)
     if ( event.RightDown() )
     {
         // find the tool under the mouse
-        wxCoord x,y;
+        wxCoord x = 0, y = 0;
         event.GetPosition(&x, &y);
 
         wxToolBarToolBase *tool = FindToolForPosition(x, y);
@@ -1370,34 +1382,48 @@ void wxToolBar::OnMouseEvent(wxMouseEvent& event)
 // colour: for example, when it must blend in with a notebook page.
 void wxToolBar::OnEraseBackground(wxEraseEvent& event)
 {
-    wxColour bgCol = GetBackgroundColour();
-    if (!bgCol.Ok())
-    {
-        event.Skip();
-        return;
-    }
-
-    // notice that this 'dumb' implementation may cause flicker for some of the
-    // controls in which case they should intercept wxEraseEvent and process it
-    // themselves somehow
-
-    RECT rect;
-    ::GetClientRect(GetHwnd(), &rect);
-
-    HBRUSH hBrush = ::CreateSolidBrush(wxColourToRGB(bgCol));
-
+    RECT rect = wxGetClientRect(GetHwnd());
     HDC hdc = GetHdcOf((*event.GetDC()));
 
-#ifndef __WXWINCE__
-    int mode = ::SetMapMode(hdc, MM_TEXT);
-#endif
+#if wxUSE_UXTHEME
+    // we may need to draw themed colour so that we appear correctly on
+    // e.g. notebook page under XP with themes but only do it if the parent
+    // draws themed background itself
+    if ( !UseBgCol() && !GetParent()->UseBgCol() )
+    {
+        wxUxThemeEngine *theme = wxUxThemeEngine::GetIfActive();
+        if ( theme )
+        {
+            HRESULT
+                hr = theme->DrawThemeParentBackground(GetHwnd(), hdc, &rect);
+            if ( hr == S_OK )
+                return;
 
-    ::FillRect(hdc, &rect, hBrush);
-    ::DeleteObject(hBrush);
+            // it can also return S_FALSE which seems to simply say that it
+            // didn't draw anything but no error really occurred
+            if ( FAILED(hr) )
+                wxLogApiError(_T("DrawThemeParentBackground(toolbar)"), hr);
+        }
+    }
+#endif // wxUSE_UXTHEME
 
-#ifndef __WXWINCE__
-    ::SetMapMode(hdc, mode);
-#endif
+    if ( UseBgCol() || (GetMSWToolbarStyle() & TBSTYLE_TRANSPARENT) )
+    {
+        // do draw our background
+        //
+        // notice that this 'dumb' implementation may cause flicker for some of
+        // the controls in which case they should intercept wxEraseEvent and
+        // process it themselves somehow
+        AutoHBRUSH hBrush(wxColourToRGB(GetBackgroundColour()));
+
+        wxCHANGE_HDC_MAP_MODE(hdc, MM_TEXT);
+        ::FillRect(hdc, &rect, hBrush);
+    }
+    else // we have no non default background colour
+    {
+        // let the system do it for us
+        event.Skip();
+    }
 }
 
 bool wxToolBar::HandleSize(WXWPARAM WXUNUSED(wParam), WXLPARAM lParam)
@@ -1409,7 +1435,7 @@ bool wxToolBar::HandleSize(WXWPARAM WXUNUSED(wParam), WXLPARAM lParam)
     {
         int w, h;
 
-        if ( GetWindowStyle() & wxTB_VERTICAL )
+        if ( IsVertical() )
         {
             w = r.right - r.left;
             if ( m_maxRows )
@@ -1448,10 +1474,10 @@ bool wxToolBar::HandleSize(WXWPARAM WXUNUSED(wParam), WXLPARAM lParam)
 
 bool wxToolBar::HandlePaint(WXWPARAM wParam, WXLPARAM lParam)
 {
-    // erase any dummy separators which we used for aligning the controls if
-    // any here
+    // erase any dummy separators which were used
+    // for aligning the controls if any here
 
-    // first of all, do we have any controls at all?
+    // first of all, are there any controls at all?
     wxToolBarToolsList::compatibility_iterator node;
     for ( node = m_tools.GetFirst(); node; node = node->GetNext() )
     {
@@ -1460,10 +1486,8 @@ bool wxToolBar::HandlePaint(WXWPARAM wParam, WXLPARAM lParam)
     }
 
     if ( !node )
-    {
         // no controls, nothing to erase
         return false;
-    }
 
     // prepare the DC on which we'll be drawing
     wxClientDC dc(this);
@@ -1472,10 +1496,8 @@ bool wxToolBar::HandlePaint(WXWPARAM wParam, WXLPARAM lParam)
 
     RECT r;
     if ( !::GetUpdateRect(GetHwnd(), &r, FALSE) )
-    {
         // nothing to redraw anyhow
         return false;
-    }
 
     wxRect rectUpdate;
     wxCopyRECTToRect(r, rectUpdate);
@@ -1553,14 +1575,14 @@ void wxToolBar::HandleMouseMove(WXWPARAM WXUNUSED(wParam), WXLPARAM lParam)
     wxToolBarToolBase* tool = FindToolForPosition( x, y );
 
     // cursor left current tool
-    if( tool != m_pInTool && !tool )
+    if ( tool != m_pInTool && !tool )
     {
         m_pInTool = 0;
         OnMouseEnter( -1 );
     }
 
     // cursor entered a tool
-    if( tool != m_pInTool && tool )
+    if ( tool != m_pInTool && tool )
     {
         m_pInTool = tool;
         OnMouseEnter( tool->GetId() );
@@ -1587,6 +1609,9 @@ WXLRESULT wxToolBar::MSWWindowProc(WXUINT nMsg, WXWPARAM wParam, WXLPARAM lParam
             if ( HandlePaint(wParam, lParam) )
                 return 0;
 #endif
+
+        default:
+            break;
     }
 
     return wxControl::MSWWindowProc(nMsg, wParam, lParam);
@@ -1596,6 +1621,8 @@ WXLRESULT wxToolBar::MSWWindowProc(WXUINT nMsg, WXWPARAM wParam, WXLPARAM lParam
 // private functions
 // ----------------------------------------------------------------------------
 
+#ifdef wxREMAP_BUTTON_COLOURS
+
 WXHBITMAP wxToolBar::MapBitmap(WXHBITMAP bitmap, int width, int height)
 {
     MemoryHDC hdcMem;
@@ -1631,7 +1658,8 @@ WXHBITMAP wxToolBar::MapBitmap(WXHBITMAP bitmap, int width, int height)
                      abs(GetGValue(pixel) - GetGValue(col)) < 10 &&
                      abs(GetBValue(pixel) - GetBValue(col)) < 10 )
                 {
-                    ::SetPixel(hdcMem, i, j, cmap[k].to);
+                    if ( cmap[k].to != pixel )
+                        ::SetPixel(hdcMem, i, j, cmap[k].to);
                     break;
                 }
             }
@@ -1639,68 +1667,8 @@ WXHBITMAP wxToolBar::MapBitmap(WXHBITMAP bitmap, int width, int height)
     }
 
     return bitmap;
-
-    // VZ: I leave here my attempts to map the bitmap to the system colours
-    //     faster by using BitBlt() even though it's broken currently - but
-    //     maybe someone else can finish it? It should be faster than iterating
-    //     over all pixels...
-#if 0
-    MemoryHDC hdcMask, hdcDst;
-    if ( !hdcMask || !hdcDst )
-    {
-        wxLogLastError(_T("CreateCompatibleDC"));
-
-        return bitmap;
-    }
-
-    // create the target bitmap
-    HBITMAP hbmpDst = ::CreateCompatibleBitmap(hdcDst, width, height);
-    if ( !hbmpDst )
-    {
-        wxLogLastError(_T("CreateCompatibleBitmap"));
-
-        return bitmap;
-    }
-
-    // create the monochrome mask bitmap
-    HBITMAP hbmpMask = ::CreateBitmap(width, height, 1, 1, 0);
-    if ( !hbmpMask )
-    {
-        wxLogLastError(_T("CreateBitmap(mono)"));
-
-        ::DeleteObject(hbmpDst);
-
-        return bitmap;
-    }
-
-    SelectInHDC bmpInDst(hdcDst, hbmpDst),
-                bmpInMask(hdcMask, hbmpMask);
-
-    // for each colour:
-    for ( n = 0; n < NUM_OF_MAPPED_COLOURS; n++ )
-    {
-        // create the mask for this colour
-        ::SetBkColor(hdcMem, ColorMap[n].from);
-        ::BitBlt(hdcMask, 0, 0, width, height, hdcMem, 0, 0, SRCCOPY);
-
-        // replace this colour with the target one in the dst bitmap
-        HBRUSH hbr = ::CreateSolidBrush(ColorMap[n].to);
-        HGDIOBJ hbrOld = ::SelectObject(hdcDst, hbr);
-
-        ::MaskBlt(hdcDst, 0, 0, width, height,
-                  hdcMem, 0, 0,
-                  hbmpMask, 0, 0,
-                  MAKEROP4(PATCOPY, SRCCOPY));
-
-        (void)::SelectObject(hdcDst, hbrOld);
-        ::DeleteObject(hbr);
-    }
-
-    ::DeleteObject((HBITMAP)bitmap);
-
-    return (WXHBITMAP)hbmpDst;
-#endif // 0
 }
 
-#endif // wxUSE_TOOLBAR && Win95
+#endif // wxREMAP_BUTTON_COLOURS
 
+#endif // wxUSE_TOOLBAR
