@@ -5,15 +5,11 @@
 // Modified by: Wlodzimiez ABX Skiba 2003/2004 Unicode support
 //              Ron Lee
 // Created:     7.9.93
-// RCS-ID:      $Id: xlputils.cpp,v 1.10 2005/08/24 17:56:29 ABX Exp $
+// RCS-ID:      $Id: xlputils.cpp,v 1.11 2005/09/23 12:56:35 MR Exp $
 // Copyright:   (c) Julian Smart
 // Licence:     wxWindows licence
 /////////////////////////////////////////////////////////////////////////////
 
-#ifdef __GNUG__
-#pragma implementation
-#endif
-
 // For compilers that support precompilation, includes "wx.h".
 #include "wx/wxprec.h"
 
